# Check-Cert.ps1
#
# run with: powershell.exe Check-Cert.ps1 -CertStore <Cert Store Path> -WarningDays <Warning Days> -CriticalDays <Critical Days>
#
# 2022-11-23 thomas.linder@green.ch initial version
#
# Deployment: C:\Scripts\Check-Cert.ps1
# Execution example: C:\Windows\System32\WindowsPowerShell\v1.0\powershell.exe C:\Scripts\Check-Cert.ps1 -CertStore \LocalMachine -WarningDays 100 -CriticalDays 5

Param (
	[String] $CertStore = "\LocalMachine\My",
	[Int] $WarningDays = 24,
	[Int] $CriticalDays = 4,
	[Int] $IgnoreDaysPast = -15, # Ignore certificates expired longer in the past
	[Int] $IgnoreDaysFuture = 1500 # Hide certificates expiring far in the future
)

function Get-StringHashB64 {
	param (
		[String] $value
	)
	# Write-Host $value
	[Convert]::ToBase64String([System.Security.Cryptography.HashAlgorithm]::Create("sha256").ComputeHash([System.Text.Encoding]::UTF8.GetBytes($value)))
}

# Write-Output "Cert Store: $CertStore, Days: $WarningDays"

Get-ChildItem -path "Cert:$CertStore" -recurse |
  where { $_.Subject `
           -AND $_.NotAfter -le (Get-Date).AddDays($IgnoreDaysFuture) `
           -AND $_.NotAfter -gt (Get-Date).AddDays($IgnoreDaysPast) } |
  Select @{name="DaysLeft";expression={ (New-TimeSpan -Start (Get-Date) -End $_.NotAfter).Days } }, `
         @{name="FolderName";expression={ Convert-Path $_.PSParentPath } }, `
		 @{name="HashB64";expression={ Get-StringHashB64($_.Subject + "|" + (Convert-Path $_.PSParentPath)) } }, `
  	     Subject, NotAfter, Thumbprint |
  Select @{name="Status";expression={ switch($_.DaysLeft) { { $_ -le $CriticalDays } { 'CRITICAL'; break; } { $_ -le $WarningDays } { 'WARNING'; break; } default { 'OK' } } } }, `
		 DaysLeft, Subject, NotAfter, FolderName, HashB64, Thumbprint |
  Foreach-Object { if ($_.Status) { "{0} {1} {2} {3} in {4} expiration: {5}" -f $_.Status, $_.HashB64, $_.DaysLeft, $_.Subject, $_.FolderName, $_.NotAfter | Write-Output } else { Write-Output "OOPS" } }
